@@ -1,7 +1,10 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- *   Gnome Apt frontend
+/** -*- Mode: C++; tab-width: 4 -*-
+ * vim: sw=4:ts=4:
  *
- *   Copyright (C) 1998 Havoc Pennington <hp@pobox.com>
+ * Gnome Apt file retrieval class
+ *
+ * 	(C) 1998 Havoc Pennington <hp@pobox.com>
+ * 	    2002, 2003 Filip Van Raemdonck <mechanix@debian.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -17,16 +20,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
- */
+ *
+ * 	$Id$
+ *
+ **/
+
 using namespace std;
 
-#include "acquirestatus.h"
+#include <stdio.h>
 #include <unistd.h>
 #include <apt-pkg/acquire-item.h>
 #include <apt-pkg/acquire-worker.h>
-#include <stdio.h>
+#include <apt-pkg/error.h>
 #include <apt-pkg/strutl.h>
+#include <gtk/gtk.h>
 
+#include "acquirestatus.h"
+#include "gnome-apt.h"
 
 gboolean 
 GAcqStatus::close_cb(GtkWidget* dialog, gpointer data)
@@ -38,62 +48,59 @@ GAcqStatus::close_cb(GtkWidget* dialog, gpointer data)
   return TRUE; /* block the close - we'll close in the destructor after the next iteration of the acquire */
 }
 
-GAcqStatus::GAcqStatus()
-  : cancelled_(false), dialog_(0), itembar_(0), overallbar_(0), label_(0)
-{
-  dialog_ = gnome_dialog_new(_("Download Progress"),
-                             GNOME_STOCK_BUTTON_CANCEL, 
-                             NULL);
-  
-  gnome_apt_setup_dialog(dialog_);  
-  gtk_window_set_modal(GTK_WINDOW(dialog_), TRUE);
-  gtk_window_set_policy(GTK_WINDOW(dialog_), TRUE, TRUE, FALSE);
-  gnome_dialog_set_close(GNOME_DIALOG(dialog_), TRUE);
-  gtk_signal_connect(GTK_OBJECT(dialog_), "close",
-                     GTK_SIGNAL_FUNC(close_cb), this);
+gboolean
+GAcqStatus::response_cb (GtkWidget* w, gint response, gpointer data) {
+	if (response == GTK_RESPONSE_CANCEL) {
+		GAcqStatus* as = static_cast<GAcqStatus*> (data);
+		as->cancelled_ = true;
+	} else {
+		g_warning ("Got response: %d", response);
+	}
 
-  gtk_widget_set_usize(dialog_, 450, 450);
+	return TRUE;
+}
+
+GAcqStatus::GAcqStatus() : cancelled_(false) {
+	dialog_ = gtk_dialog_new_with_buttons (
+		_("Download Progress"), NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+	gtk_window_set_default_size (GTK_WINDOW (dialog_), 450, 450);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog_), GTK_RESPONSE_CANCEL);
+
+  gnome_apt_setup_dialog(dialog_);  
+	g_signal_connect (
+		G_OBJECT (dialog_), "close", G_CALLBACK (close_cb), this);
+	g_signal_connect (
+		G_OBJECT (dialog_), "response", G_CALLBACK (response_cb), this);
 
   itembar_    = gtk_progress_bar_new();
 
-  gtk_progress_set_format_string(GTK_PROGRESS(itembar_), _("Active download(s) %P%%"));
-  gtk_progress_set_show_text(GTK_PROGRESS(itembar_), TRUE);
+	/*gtk_progress_bar_set_text (GTK_PROGRESS_BAR (itembar_), _("Active downloads(s) %P%%"));*/
 
   overallbar_ = gtk_progress_bar_new();
 
-  gtk_progress_set_format_string(GTK_PROGRESS(overallbar_), _("Overall %P%%"));
-  gtk_progress_set_show_text(GTK_PROGRESS(overallbar_), TRUE);
-  gtk_progress_bar_set_activity_blocks(GTK_PROGRESS_BAR(overallbar_), 10);
+	/*gtk_progress_bar_set_text (GTK_PROGRESS_BAR (overallbar_), _("Overall %P%%"));*/
+	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (overallbar_), 0.1);
 
   label_ = gtk_label_new("");
   etalabel_ = gtk_label_new("");
 
   GtkWidget* frame = gtk_frame_new(_("Error Log"));
 
-  errors_ = gtk_text_new(0,0);
-  gtk_widget_set_usize(errors_, -1, 100);
+	errors_ = gtk_text_view_new ();
+	gtk_widget_set_size_request (errors_, -1, 100);
+	gtk_container_add (GTK_CONTAINER (frame), errors_);
 
-  gtk_container_add(GTK_CONTAINER(frame), errors_);
-
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog_)->vbox),
-                     label_, 
-                     TRUE, TRUE, GNOME_PAD);
-
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog_)->vbox),
-                     etalabel_, 
-                     TRUE, TRUE, GNOME_PAD);
-
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog_)->vbox),
-                     frame, 
-                     FALSE, FALSE, GNOME_PAD_SMALL);  
-
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog_)->vbox),
-                     itembar_, 
-                     FALSE, FALSE, GNOME_PAD_SMALL);
-
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog_)->vbox),
-                     overallbar_, 
-                     FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_)->vbox), label_,
+	      TRUE, TRUE, GAPT_PAD);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_)->vbox), etalabel_,
+	      TRUE, TRUE, GAPT_PAD);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_)->vbox), frame,
+	      FALSE, FALSE, GAPT_PAD);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_)->vbox), itembar_,
+	      FALSE, FALSE, GAPT_PAD);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_)->vbox), overallbar_,
+	      FALSE, FALSE, GAPT_PAD);
 
    // flush
   while (gtk_events_pending())
@@ -107,13 +114,12 @@ GAcqStatus::Start()
    ID = 1;
    cancelled_ = false;
 
-   cerr << "Start" << endl;
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (itembar_), 0.0);
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (overallbar_), 0.0);
 
-   gtk_progress_set_percentage(GTK_PROGRESS(itembar_), 0.0);
-   gtk_progress_set_percentage(GTK_PROGRESS(overallbar_), 0.0);
-
-   gtk_editable_delete_text(GTK_EDITABLE(errors_), 
-                            0, gtk_text_get_length(GTK_TEXT(errors_)));
+	gtk_text_buffer_set_text (
+		GTK_TEXT_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (errors_))),
+		"", -1);
 
    gtk_widget_show_all(dialog_);
 };
@@ -121,20 +127,12 @@ GAcqStatus::Start()
 bool 
 GAcqStatus::MediaChange(string Media,string Drive)
 {
-  
-  gchar buf[256];
-
-  g_snprintf(buf,255,_("Please insert:\n'%s'\nin drive %s"),
-             Media.c_str(), Drive.c_str());
-
-  GtkWidget* subdialog = gnome_ok_dialog(buf);
-  gnome_dialog_set_parent(GNOME_DIALOG(subdialog), GTK_WINDOW(dialog_));
-
-  // block until clicked
-  gnome_dialog_run(GNOME_DIALOG(subdialog));
-   
-  // Hope apt-pkg pops the dialog again if the media isn't really
-  //  there?
+	GtkWidget* subdialog = gtk_message_dialog_new (
+		NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+		_("Please insert:\n'%s'\nin drive %s"), Media.c_str(), Drive.c_str());
+	gtk_window_set_transient_for (GTK_WINDOW (subdialog), GTK_WINDOW (dialog_));
+	gtk_dialog_run (GTK_DIALOG (subdialog));
+	gtk_widget_destroy (subdialog);
 
   Update = true;
   return true;
@@ -149,14 +147,13 @@ GAcqStatus::IMSHit(pkgAcquire::ItemDesc &Itm)
   cerr << "Hit " << Itm.Description << endl;
 
   int n = g_snprintf(buf, 255, _("Already have %s "), Itm.Description.c_str());
-  if (n > 0) {
-    if (Itm.Owner->FileSize != 0)
-      g_snprintf(&buf[n], 255-n, _("[%s]"), SizeToStr(Itm.Owner->FileSize).c_str());
-  }
-  
-  gtk_label_set(GTK_LABEL(label_), buf);
-  
-  // flush
+	if (n > 0) {
+		if (Itm.Owner->FileSize) {
+			g_snprintf (&buf[n], 255 - n, "[%s]", SizeToStr (Itm.Owner->FileSize).c_str());
+		}
+	}
+	gtk_label_set_text (GTK_LABEL (label_), buf);
+
   while (gtk_events_pending())
     gtk_main_iteration();
 
@@ -179,10 +176,10 @@ GAcqStatus::Fetch(pkgAcquire::ItemDesc &Itm)
   gchar buf[256];
 
 #if 0
-  int n = g_snprintf(buf, 255, _("%s    %%P%%"), Itm.Description.c_str());
+  int n = g_snprintf (buf, 255, "%s    %%P%%", Itm.Description.c_str());
   if (n > 0) {
     if (Itm.Owner->FileSize != 0)
-      g_snprintf(&buf[n], 255-n, _("/%s"), SizeToStr(Itm.Owner->FileSize).c_str());
+      g_snprintf (&buf[n], 255-n, "/%s", SizeToStr (Itm.Owner->FileSize).c_str());
   }
 
    gtk_progress_set_format_string(GTK_PROGRESS(itembar_), buf);
@@ -192,8 +189,7 @@ GAcqStatus::Fetch(pkgAcquire::ItemDesc &Itm)
    g_snprintf(buf, 255, _("Get: %ld %s [%s]"),
               Itm.Owner->ID, Itm.Description.c_str(),
               SizeToStr(Itm.Owner->FileSize).c_str());
-
-   gtk_label_set(GTK_LABEL(label_), buf);
+	gtk_label_set_text (GTK_LABEL (label_), buf);
 
    // flush
    while (gtk_events_pending())
@@ -208,8 +204,7 @@ GAcqStatus::Done(pkgAcquire::ItemDesc &Itm)
   gchar buf[256];
   g_snprintf(buf, 255, _("Finished %s"), 
              Itm.Description.c_str());
-
-  gtk_label_set(GTK_LABEL(label_), buf);
+	gtk_label_set_text (GTK_LABEL (label_), buf);
 
    // flush
   while (gtk_events_pending())
@@ -223,21 +218,22 @@ void
 GAcqStatus::Fail(pkgAcquire::ItemDesc &Itm)
 {   
   gchar buf[256];
+	GtkTextBuffer* buffer;
+	GtkTextIter pend;
+
+	gchar* errmsg = g_strdup_printf (_("Failed to fetch %s\n  %s\n"),
+	      Itm.Owner->DescURI().c_str(), Itm.Owner->ErrorText.c_str());
+	cerr << errmsg;
+	_error->Warning (errmsg);
 
-  cerr << "Fail " << Itm.Description << endl;
-  
   g_snprintf(buf, 255, _("%s\n%s"), 
              Itm.Description.c_str(), 
              Itm.Owner->ErrorText.c_str());
+	gtk_label_set_text (GTK_LABEL (label_), buf);
 
-  gtk_label_set(GTK_LABEL(label_), buf);
-
-  gint pos = gtk_text_get_length(GTK_TEXT(errors_));
-
-  gtk_editable_insert_text(GTK_EDITABLE(errors_),
-                           buf,
-                           strlen(buf),
-                           &pos);
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (errors_));
+	gtk_text_buffer_get_end_iter (buffer, &pend);
+	gtk_text_buffer_insert (buffer, &pend, buf, -1);
 
 #if 0
   gtk_progress_set_show_text(GTK_PROGRESS(itembar_), FALSE);
@@ -267,9 +263,8 @@ GAcqStatus::Stop()
                SizeToStr(CurrentCPS).c_str());
   else
     g_snprintf(buf, 255, _("Fetched no data."));
+	gtk_label_set_text (GTK_LABEL (label_), buf);
 
-  gtk_label_set(GTK_LABEL(label_), buf);
-  
   // flush
   while (gtk_events_pending())
     gtk_main_iteration();
@@ -316,13 +311,14 @@ GAcqStatus::Pulse(pkgAcquire *Owner)
       
      gchar buf[512];
 
-      // Add in the short description
-     if (i->CurrentItem->Owner->ID != 0)
-       g_snprintf(buf,511, _("Item %lx, %s"),
-                  i->CurrentItem->Owner->ID,
-                  i->CurrentItem->ShortDesc.c_str());
-     else
-       g_snprintf(buf,511,"%s",i->CurrentItem->ShortDesc.c_str());
+		// Add in the short description
+		if (i->CurrentItem->Owner->ID != 0) {
+			g_snprintf (buf, 511, _("Item %ld, %s"),
+				i->CurrentItem->Owner->ID,
+				i->CurrentItem->ShortDesc.c_str());
+		} else {
+			g_snprintf (buf, 511, "%s", i->CurrentItem->ShortDesc.c_str());
+		}
 
      s += buf;
 
@@ -356,34 +352,20 @@ GAcqStatus::Pulse(pkgAcquire *Owner)
      i = Owner->WorkerStep(i);
    }
 
-   gtk_progress_set_activity_mode(GTK_PROGRESS(overallbar_), !Shown);
-   gtk_progress_set_activity_mode(GTK_PROGRESS(itembar_), !Shown);
-
    if (!Shown) {
-     gfloat new_val;
-     GtkAdjustment *adj;
-     
-     adj = GTK_PROGRESS(overallbar_)->adjustment;
-     
-     new_val = adj->value + 1;
-     if (new_val > adj->upper)
-       new_val = adj->lower;
+		gtk_progress_bar_pulse (GTK_PROGRESS_BAR (overallbar_));
 
-     gtk_progress_set_value (GTK_PROGRESS (overallbar_), new_val);
-
-     gtk_label_set(GTK_LABEL(label_), _("No active downloads"));
-   }
-   else {
-     // Set label
-     string list_of_workers = _("Active download(s):\n");
+		gtk_label_set_text (GTK_LABEL (label_), _("No active downloads"));
+	} else {
+		string list_of_workers = _("Active download(s):\n");
      vector<string>::iterator q = workers.begin();
      while (q != workers.end()) {
        list_of_workers += *q;
        list_of_workers += "\n";
        ++q;
      }
-     gtk_label_set(GTK_LABEL(label_), list_of_workers.c_str());
-     
+		gtk_label_set_text (GTK_LABEL (label_), list_of_workers.c_str());
+
      float workerpercent = 0.0;
      if (workers_total != 0) 
        workerpercent = float(workers_current)/float(workers_total);
@@ -391,10 +373,10 @@ GAcqStatus::Pulse(pkgAcquire *Owner)
        workerpercent = 0.0;
 
      // Put in the percent done, clamping to valid values.
-     gtk_progress_set_percentage(GTK_PROGRESS(itembar_), 
-                                 CLAMP(workerpercent, 0.0, 1.0));
-     gtk_progress_set_percentage(GTK_PROGRESS(overallbar_), 
-                                 CLAMP(float(CurrentBytes + CurrentItems)/float(TotalBytes+TotalItems), 0.0, 1.0));
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (itembar_),
+		      CLAMP (workerpercent, 0.0, 1.0));
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (overallbar_), 
+		      CLAMP (float (CurrentBytes + CurrentItems) / float (TotalBytes + TotalItems), 0.0, 1.0));
    }
       
    // Put in the ETA and cps meter
@@ -405,10 +387,10 @@ GAcqStatus::Pulse(pkgAcquire *Owner)
        g_snprintf(Tmp,299,_("%s remaining at %s/s"),
                   TimeToStr(ETA).c_str(), 
                   SizeToStr(CurrentCPS).c_str());
-       gtk_label_set(GTK_LABEL(etalabel_), Tmp);
+		gtk_label_set_text (GTK_LABEL (etalabel_), Tmp);
      }
    else 
-     gtk_label_set(GTK_LABEL(etalabel_), _("Stalled"));
+		gtk_label_set_text (GTK_LABEL (etalabel_), _("Stalled"));
  
    // flush
    while (gtk_events_pending())
