@@ -1,10 +1,10 @@
 /** -*- Mode: C++; tab-width: 4 -*-
- * vim: sw=4:ts=4:
+ * vim: sw=4 ts=4:
  *
  * Gnome Apt package installation handler
  *
  * 	(C) 1998 Havoc Pennington <hp@pobox.com>
- * 	    2003 Filip Van Raemdonck <mechanix@debian.org>
+ * 	    2003-2005 Filip Van Raemdonck <mechanix@debian.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,19 +30,20 @@
 
 #include <gtk/gtkwidget.h>
 
+typedef struct _GAptCacheControl GAptCacheControl;
+class pkgPackageManager;
+
 class ChildDialog {
+/* Ugh... this class would better be converted into a gobject so it can emit it's own signals */
+friend bool gapt_cache_control_install (GAptCacheControl*, pkgPackageManager*);
 public:
-  
-  ChildDialog(const char* title,
-              const char* label);
-  ~ChildDialog();
+	ChildDialog (const char*, const char*);
+	~ChildDialog (void);
 
-  int fork();
-
-  // Parent should so this, child should exec() or whatever;
-  // return false if the child died abnormally
-  //  or close was clicked.
-  bool run();
+	gint child_died (gint, gint);
+	int fork (void);
+	gint run (void);
+	void feed (const gchar*);
 
 private:
   GtkWidget* dialog_;
@@ -50,14 +51,12 @@ private:
   GtkWidget* label_;
   bool we_are_parent_;
 
-  bool child_done_;
-  bool close_anyway_;
+	bool child_done_;
+	gint pid;
+	gint res;
 
-  static void child_died_cb(GtkWidget* w, gpointer data);
-  void child_died();
-
-  static gint close_cb(GtkWidget* w, gpointer data);
-  gint close();
+	static gint close_cb (GtkWidget*, gpointer);
+	gint close (void);
 };
 
 #endif
