@@ -1,7 +1,10 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- *   Gnome Apt frontend
+/** -*- Mode: C++; tab-width: 4 -*-
+ * vim: sw=4 ts=4:
  *
- *   Copyright (C) 1998 Havoc Pennington <hp@pobox.com>
+ * Gnome Apt progress indicator dialog class
+ *
+ * 	(C) 1998 Havoc Pennington <hp@pobox.com>
+ * 	    2003 Filip Van Raemdonck <mechanix@debian.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -17,32 +20,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
- */
+ *
+ * 	$Id$
+ *
+ **/
 
 #ifndef GNOME_APT_DIALOGPROGRESS_H
 #define GNOME_APT_DIALOGPROGRESS_H
 
+#include <gtk/gtkwidget.h>
 #include <apt-pkg/progress.h>
-#include "gnome-apt.h"
 
-class OpDialogProgress : public OpProgress
-{
+class OpDialogProgress : public OpProgress {
 private:
   GtkWidget* dialog_;
   GtkWidget* bar_;
   GtkWidget* label_;
  
 protected:
-  
   virtual void Update();
   
 public:
-  
-  virtual void Done();
-  
-  OpDialogProgress(const char* title); 
-  virtual ~OpDialogProgress();
+	OpDialogProgress (GtkWidget*);
+
+	virtual void Done (void);
+	OpDialogProgress* This (void) { return this; }
 };
 
-
 #endif
