@@ -4,7 +4,7 @@
  * Gnome Apt package tree display
  *
  * 	(C) 1998 Havoc Pennington <hp@pobox.com>
- * 	    2002-2004 Filip Van Raemdonck <mechanix@debian.org>
+ * 	    2002-2005 Filip Van Raemdonck <mechanix@debian.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,8 +34,10 @@ using namespace std;
 #include <algorithm>
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-config.h>
 
+#include <cstring>
+
+#include "conffile.h"
 #include "drawtree.h"
 #include "gaptdrawingarea.h"
 #include "gaptpixbuf.h"
@@ -179,7 +181,7 @@ DrawTree::scroll_event (GdkEventScroll* ev) {
 			}
 		}
 		scroll_by (step);
-		recalc_rows();
+		do_recalc_rows();
 		return TRUE;
 	}
 	return FALSE;
@@ -217,9 +219,9 @@ DrawTree::configure(GdkEventConfigure* event)
                            pixwidth_,
                            pixheight_,
                            -1);
-		   
-  recalc_widths(); 
-  recalc_rows();
+
+	recalc_widths();
+	do_recalc_rows();
 
   setup_adjustments(); // page size will have changed
 
@@ -478,8 +480,6 @@ DrawTree::key (GdkEventKey* event) {
 	return ret;
 }
 
-static void save_column_widths (const vector<DrawTree::Column*>);
-
 static gboolean
 in_collection (Nodes& coll, TreeNode* node) {
 	Nodes::iterator i = coll.begin();
@@ -494,9 +494,9 @@ in_collection (Nodes& coll, TreeNode* node) {
 	return FALSE;
 }
 
+static void save_column_widths (const vector<DrawTree::Column*>, ConfFile*);
 gint
-DrawTree::motion(GdkEventMotion* event)
-{
+DrawTree::motion (GdkEventMotion* event) {
 #if 0
   // This should be in enter_notify, I'm just lazy for now
   if (!GTK_WIDGET_HAS_FOCUS(da_)) 
@@ -560,13 +560,13 @@ DrawTree::motion(GdkEventMotion* event)
 
       left->width = newwidth;
 
-		save_column_widths (columns_);
-      recalc_widths();
-      draw_resize_line();
-      return true;
-    }
+		save_column_widths (columns_, conffile);
+		recalc_widths();
+		draw_resize_line();
+		return true;
+	}
 
-  return false;
+	return false;
 }
 
 gint 
@@ -881,7 +881,7 @@ DrawTree::change_selection (Node* newnode, bool shutdown) {
 int
 DrawTree::focus_node (Node* node, bool scroll) {
 	focus_node_ = node;
-	recalc_rows();
+	do_recalc_rows();
 	focus_node_ = 0;
 
 	int scrollby = (int) focus_index_ - (int) row_;
@@ -1058,43 +1058,34 @@ find_column (vector<DrawTree::Column*> cols, DrawTree::ColumnType t) {
 }
 
 static void
-load_column_widths (vector<DrawTree::Column*>& cols) {
-	gpointer config_iterator;
+load_column_widths (vector<DrawTree::Column*>& cols, ConfFile* cf) {
 	guint loaded = 0;
+	gsize len = 0;
+	const gchar* group = "ColumnWidths";
 
-	config_iterator = gnome_config_init_iterator ("/gnome-apt/ColumnWidths");
+	gchar** keys = cf->get_keys (group, &len);
+	if (keys) {
+		while (*keys) {
+			gint width = cf->get_integer (group, *keys);
 
-	if (config_iterator) {
-		gchar *col, *width;
-
-		loaded = 0;
-		while ((config_iterator = gnome_config_iterator_next (config_iterator, &col, &width))) {
-			if (!width || !col) {
-				g_free (width);
-				g_free (col);
-				continue;
-			}
-
-			DrawTree::Column* found = find_column (cols, string_to_column (col));
+			DrawTree::Column* found = find_column (cols, string_to_column (*keys));
 			if (found) {
-				gint wid = atoi (width);
-				found->setWidth (wid);
+				found->setWidth (width);
 			}
-			g_free (width); width = 0;
-			g_free (col); col = 0;
 
 			++loaded;
+			++keys;
 		}
 	}
 
-	if (loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
+	if (len != loaded || loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
 		/* Maybe the user just changed gnome-apt versions, or we are otherwise
 		 * weirded up.
 		 * Clean the section - otherwise an old entry could remain forever and
 		 * keep screwing us up in the future.
 		 */
-		gnome_config_clean_section ("/gnome-apt/ColumnWidths");
-		gnome_config_sync();
+		cf->clean_section (group);
+		cf->sync();
 
 		int i = 0;
 		while (i < DrawTree::ColumnTypeEnd) {
@@ -1121,90 +1112,70 @@ load_column_widths (vector<DrawTree::Column*>& cols) {
 
 /* FIXME: collapse load_column_* functions */
 static void
-load_column_visibility (vector<DrawTree::Column*>& cols) {
-	gpointer config_iterator;
+load_column_visibility (vector<DrawTree::Column*>& cols, ConfFile* cf) {
 	guint loaded = 0;
+	gsize len = 0;
+	const gchar* group = "ColumnVisibility";
 
-	config_iterator = gnome_config_init_iterator ("/gnome-apt/ColumnVisibility");
+	gchar** keys = cf->get_keys (group, &len);
+	if (keys) {
+		while (*keys) {
+			gboolean vis = cf->get_bool (group, *keys);
 
-	if (config_iterator) {
-		gchar *col, *visibility;
-
-		loaded = 0;
-		while ((config_iterator = gnome_config_iterator_next (config_iterator, &col, &visibility))) {
-			if (!visibility || !col) {
-				g_free (visibility);
-				g_free (col);
-				continue;
-			}
-
-			DrawTree::Column* found = find_column (cols, string_to_column (col));
+			DrawTree::Column* found = find_column (cols, string_to_column (*keys));
 			if (found) {
-				if (!g_ascii_strcasecmp (g_strstrip (visibility), "false")) {
-					found->setVisible (false);
-				}
+				found->setVisible (vis);
 			}
-			g_free (visibility); visibility = 0;
-			g_free (col); col = 0;
 
 			++loaded;
+			++keys;
 		}
 	}
 
-	if (loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
+	if (len != loaded || loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
 		/* Maybe the user just changed gnome-apt versions, or we are otherwise
 		 * weirded up.
 		 * Clean the section - otherwise an old entry could remain forever and
 		 * keep screwing us up in the future.
 		 */
-		gnome_config_clean_section ("/gnome-apt/ColumnVisibility");
-		gnome_config_sync();
+		cf->clean_section (group);
+		cf->sync();
 
 		/* No need to set defaults, the Column constructor does so */
 	}
 }
 
 static void
-load_column_order (vector<DrawTree::ColumnType>& columns) {
-	gpointer config_iterator;
+load_column_order (vector<DrawTree::ColumnType>& columns, ConfFile* cf) {
 	guint loaded = 0;
+	gsize len = 0;
+	const gchar* group = "ColumnOrder";
 
-	config_iterator = gnome_config_init_iterator ("/gnome-apt/ColumnOrder");
-
-	if (config_iterator) {
-		gchar *col, *pos;
+	gchar** keys = cf->get_keys (group, &len);
+	if (keys) {
 		columns.reserve (DrawTree::ColumnTypeEnd);
-
-		loaded = 0;
-		while ((config_iterator = gnome_config_iterator_next (config_iterator, &col, &pos))) {
-			if (!pos || !col) {
-				g_free (pos);
-				g_free (col);
-				continue;
-			}
-
-			DrawTree::ColumnType ct = string_to_column (col);
-			gint index = atoi (pos);
-
-			g_free (pos); pos = 0;
-			g_free (col); col = 0;
+		while (*keys) {
+			gint index = cf->get_integer (group, *keys);
 
 			/* This can happen if the user mangles the config file */
 			if (static_cast<guint> (index) >= columns.size()) continue;
 
+			DrawTree::ColumnType ct = string_to_column (*keys);
 			columns[index] = ct;
+
 			++loaded;
+			++keys;
 		}
 	}
 
-	if (loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
+	if (len != loaded || loaded != static_cast<guint> (DrawTree::ColumnTypeEnd)) {
 		/* Either there was no saved order, or something is busted - use
 		 * default order.
 		 * Clean the section - otherwise an old entry could remain forever and
 		 * keep screwing us up in the future.
 		 */
-		gnome_config_clean_section ("/gnome-apt/ColumnOrder");
-		gnome_config_sync();
+		cf->clean_section (group);
+		cf->sync();
 
 		ga_debug ("clearing order, got only %d (wanted %d)", loaded, DrawTree::ColumnTypeEnd);
 		columns.clear();
@@ -1228,33 +1199,29 @@ column_to_string(DrawTree::ColumnType ct) {
 }
 
 static void
-save_column_visibility (const vector<DrawTree::Column*> cols) {
+save_column_visibility (const vector<DrawTree::Column*> cols, ConfFile* cf) {
 	vector<DrawTree::Column*>::const_iterator i = cols.begin();
 	while (i != cols.end()) {
-		const gchar* s = column_to_string ((*i)->getType());
-
-		if (!s) {
+		const gchar* key = column_to_string ((*i)->getType());
+		if (!key) {
 			g_warning ("Failed to get column label for type %d - this shouldn't happen!", (*i)->getType());
 			continue;
 		}
 
-		gchar key[256];
-		g_snprintf (key, 255, "/gnome-apt/ColumnVisibility/%s", s);
-		gnome_config_set_bool (key, (*i)->isVisible());
-
+		cf->set_bool ("ColumnVisibility", key, (*i)->isVisible());
 		++i;
 	}
 
-	gnome_config_sync();
+	cf->sync();
 }
 
 static void
-save_column_widths (const vector<DrawTree::Column*> cols) {
+save_column_widths (const vector<DrawTree::Column*> cols, ConfFile* cf) {
 	vector<DrawTree::Column*>::const_iterator i = cols.begin();
 	while (i != cols.end()) {
-		const gchar* s = column_to_string ((*i)->getType());
+		const gchar* key = column_to_string ((*i)->getType());
 
-		if (!s) {
+		if (!key) {
 			g_warning ("Failed to get column label for type %d - this shouldn't happen!", (*i)->getType());
 			continue;
 		}
@@ -1262,9 +1229,7 @@ save_column_widths (const vector<DrawTree::Column*> cols) {
 #ifdef GNOME_ENABLE_DEBUG
 		ga_debug ("Saving column width %d for %s", (*i)->getWidth());
 #endif
-		gchar key[256];
-		g_snprintf (key, 255, "/gnome-apt/ColumnWidths/%s", s);
-		gnome_config_set_int (key, (*i)->getWidth());
+		cf->set_integer ("ColumnWidths", key, (*i)->getWidth());
 #ifdef GNOME_ENABLE_DEBUG_ANNOYING
 		ga_debug ("Saved %s - %d\n", key, (*i)->getWidth());
 #endif
@@ -1272,7 +1237,7 @@ save_column_widths (const vector<DrawTree::Column*> cols) {
 		++i;
 	}
 
-	gnome_config_sync();
+	cf->sync();
 }
 
 static void
@@ -1292,8 +1257,8 @@ selection_lost_cb (GObject* obj, DrawTree* tree) {
 	}
 }
 
-DrawTree::DrawTree (GAptPkgTree* tree) :
-      dt_model (tree), da_ (0), pixmap_ (0),
+DrawTree::DrawTree (GAptPkgTree* tree, ConfFile* cf) :
+      dt_model (tree), conffile (cf), da_ (0), pixmap_ (0),
       update_queued_ (false), dt_coltypes (ColumnTypeEnd), focus_node_ (0),
       focus_index_ (0), total_width_ (0), recalc_rows_queued_ (false), row_ (0),
       nrows_ (0), selected_node_ (0), dt_highlight (0), prelight_ (-1), gc_ (0),
@@ -1334,7 +1299,7 @@ DrawTree::DrawTree (GAptPkgTree* tree) :
 
 	drawtree_signals[DT_SELECTION_CHANGED] = g_signal_lookup ("selection-changed", GAPT_DRAWING_AREA_TYPE);
 
-	load_column_order (dt_coltypes);
+	load_column_order (dt_coltypes, cf);
 	load_column_attrs();
 
 	g_signal_connect (G_OBJECT (tree->gaptObject()), "model-changed", G_CALLBACK (model_changed_cb), this);
@@ -1350,8 +1315,8 @@ DrawTree::load_column_attrs (void) {
 		ga_debug ("have column %ud: %s", i, columns_[i]->getName());
 	}
 #endif
-	load_column_widths (columns_);
-	load_column_visibility (columns_);
+	load_column_widths (columns_, conffile);
+	load_column_visibility (columns_, conffile);
 	recalc_widths();
 	queue_display_update();
 }
@@ -1373,53 +1338,47 @@ DrawTree::~DrawTree (void) {
     ++i;
   }
 
-  if (recalc_idle_id_ != 0)
-    gtk_idle_remove(recalc_idle_id_);
-
+	if (recalc_idle_id_ != 0) {
+		g_source_remove (recalc_idle_id_);
+	}
 	if (update_idle_id_) {
-		gtk_idle_remove (update_idle_id_);
+		g_source_remove (update_idle_id_);
 	}
 }
 
 gint 
-DrawTree::update_idle(gpointer data)
-{
-  DrawTree* tree = static_cast<DrawTree*>(data);
-  g_return_val_if_fail(tree != 0, FALSE);
+DrawTree::update_idle (gpointer data) {
+	DrawTree* tree = static_cast<DrawTree*> (data);
+	g_return_val_if_fail (tree != 0, FALSE);
 
-  tree->do_display_update();
+	tree->do_display_update();
 
   tree->update_idle_id_ = 0;
 	ga_debug ("update_idle done");
-  return FALSE; // remove idle
+	return FALSE;
 }
 
-void 
-DrawTree::queue_display_update()
-{
-  if (!update_queued_) { 
-    update_idle_id_ = gtk_idle_add(update_idle, this); 
-    update_queued_ = true;
-  }
+void
+DrawTree::queue_display_update (void) {
+	if (!update_queued_) {
+		update_idle_id_ = g_idle_add (update_idle, this);
+		update_queued_ = true;
+	}
 }
 
 
-gint 
-DrawTree::recalc_rows_idle(gpointer data)
-{
-  DrawTree* tree = static_cast<DrawTree*>(data);
-  g_return_val_if_fail(tree != 0, FALSE);
-
+gint
+DrawTree::recalc_rows_idle (gpointer data) {
+	DrawTree* tree = static_cast<DrawTree*> (data);
+	g_return_val_if_fail (tree != 0, FALSE);
   // If recalc_rows got called in between adding the idle
   //  and calling the idle, just return. There's a race
   //  condition with display update that can cause this.
   if (!tree->recalc_rows_queued_) return FALSE; 
 
-  tree->recalc_rows();
-
-  tree->recalc_idle_id_ = 0;
-
-  return FALSE; // remove idle
+	tree->do_recalc_rows();
+	tree->recalc_idle_id_ = 0;
+	return FALSE;
 }
 
 void
@@ -1439,23 +1398,21 @@ DrawTree::queue_recalc_rows (bool statechanged) {
 	}
 
 	if (!recalc_rows_queued_) {
-    recalc_idle_id_ = gtk_idle_add(recalc_rows_idle, this); 
-    recalc_rows_queued_ = true;
-  }
+		recalc_idle_id_ = g_idle_add (recalc_rows_idle, this);
+		recalc_rows_queued_ = true;
+	}
 
 	queue_display_update();
 }
 
-void 
-DrawTree::do_display_update()
-{
-  update_queued_ = false;
-
-  if (recalc_rows_queued_) {
-    // This saves us from a race condition where we get here without
-    // recalcing the rows.
-    recalc_rows();
-  }
+void
+DrawTree::do_display_update (void) {
+	update_queued_ = false;
+	if (recalc_rows_queued_) {
+		// This saves us from a race condition where we get here without
+		// recalcing the rows.
+		do_recalc_rows();
+	}
 
   gc_ = da_->style->black_gc;
 
@@ -1532,7 +1489,7 @@ DrawTree::set_visible (ColumnType ct, bool visibility) {
 	g_return_if_fail (col < columns_.size());
 	columns_[col]->setVisible (visibility);
 
-	save_column_visibility (columns_);
+	save_column_visibility (columns_, conffile);
 	recalc_widths();
 	queue_display_update();
 }
@@ -1685,9 +1642,8 @@ DrawTree::exact_rows_onscreen (void) {
 }
 
 void
-DrawTree::recalc_rows()
-{
-  recalc_rows_queued_ = false;
+DrawTree::do_recalc_rows (void) {
+	recalc_rows_queued_ = false;
 
   visible_nodes_.clear();
   depths_.clear();
@@ -1727,11 +1683,11 @@ DrawTree::recalc_rows()
 
   g_assert(depths_.size() == visible_nodes_.size());
   g_assert(visible_nodes_.size() <= max);
-
 #ifdef GNOME_ENABLE_DEBUG
-  ga_debug(" ** recalc_rows: top: %u, visible: %u  total: %u\n", 
+	ga_debug (" ** do_recalc_rows: top: %u, visible: %u  total: %u\n", 
           row_, visible_nodes_.size(), nrows_);
 #endif
+
 }
 
 bool
@@ -1925,23 +1881,19 @@ DrawTree::vadjustment_changed()
 }
 
 static void
-save_column_order (const vector<DrawTree::ColumnType>& columns) {
+save_column_order (const vector<DrawTree::ColumnType>& columns, ConfFile* cf) {
 	g_return_if_fail (columns.size() == static_cast<guint> (DrawTree::ColumnTypeEnd));
 
 	int position = 0;
 	vector<DrawTree::ColumnType>::const_iterator i = columns.begin();
 	while (i != columns.end()) {
-		gchar key[256];
-		g_snprintf (key, 255, "/gnome-apt/ColumnOrder/%s", column_to_string (*i));
-		gchar val[30];
-		g_snprintf (val, 29, "%d", position);
-		gnome_config_set_string (key, val);
+		cf->set_integer ("ColumnOrder", column_to_string (*i), position);
 
 		++position;
 		++i;
 	}
 
-	gnome_config_sync();
+	cf->sync();
 }
 
 gushort
@@ -1960,7 +1912,7 @@ DrawTree::set_column_order (const vector<ColumnType>& cts) {
 	g_return_if_fail (cts.size() == dt_coltypes.size());
 
 	dt_coltypes = cts;
-	save_column_order (dt_coltypes);
+	save_column_order (dt_coltypes, conffile);
 
 	load_column_attrs();
 }
