@@ -4,7 +4,7 @@
  * Gnome Apt package cache handling class
  *
  * 	(C) 1998 Havoc Pennington <hp@pobox.com>
- * 	    2003 Filip Van Raemdonck <mechanix@debian.org>
+ * 	    2003-2005 Filip Van Raemdonck <mechanix@debian.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,8 +28,6 @@
 #ifndef GNOME_APT_PKGLIST_H
 #define GNOME_APT_PKGLIST_H
 
-#include <libgnomeui/gnome-appbar.h>
-
 #include "cachecontrol.h"
 #include "details.h"
 
@@ -39,7 +37,7 @@ class DrawTree;
 
 class GAptPkgList {
 public:
-	GAptPkgList (DrawTree*, GnomeAppBar*);
+	GAptPkgList (DrawTree*, gboolean);
 	~GAptPkgList (void);
 
 	GtkWidget* widget (void) { return pl_wdg; }
